"""
palaver definition: an unnecessarily elaborate or complex procedure
(A.K.A. CASA in a nutshell - fight me)
"""
import argparse
import logging
import pathlib
import subprocess
import sys
import os
import json
from typing import Dict
from pycapo import CapoConfig

from casa_envoy.auditor import AuditDirectories, AuditFiles
from casa_envoy.launchers import CalibrationLauncher, ImagingLauncher

"""
Launch CASA processing jobs via Workspaces
"""

logger = logging.getLogger("casa_envoy")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def _get_settings(cwd: pathlib.Path):
    use_casa = CapoConfig().getboolean("edu.nrao.archive.workspaces.ProcessingSettings.useCasa")
    casa_home = (
        CapoConfig().settings("edu.nrao.archive.workflow.config.CasaVersions").homeForReprocessing
    )

    root_dir = str(cwd.parent)
    processing_dir = str(cwd.stem)

    return {
        "useCasa": use_casa,
        "homeForReprocessing": casa_home,
        "rootDirectory": root_dir,
        "processingDirectory": processing_dir,
    }


def arg_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Workspaces CASA processing launcher",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "--standard-cal",
        nargs=2,
        action="store",
        required=False,
        help="run the standard calibration CASA pipeline",
    )
    parser.add_argument(
        "--standard-img",
        nargs=2,
        action="store",
        required=False,
        help="run the standard calibration CASA pipeline",
    )
    return parser


def check_calibratable(metadata_filename: str) -> bool:
    with open(metadata_filename) as json_meta:
        metadata = json.loads(json_meta.read())
        spl = metadata["productLocator"]
        if "execblock" in spl:
            return True
        else:
            logger.info("SPL ERROR: This product locator is not calibratable!")
            return False


def check_imagable(metadata_filename: str) -> bool:
    with open(metadata_filename) as json_meta:
        metadata = json.loads(json_meta.read())
        cms_name = metadata["cmsName"]
        cms_path = metadata["calibrationSourceDirectory"]
        if cms_name is not None and cms_path is not None and cms_name[-3:] == ".ms":
            return True
        else:
            logger.info("CMS ERROR: Imaging requires a valid CMS name and location!")
            return False


def run_audit(ppr: str, metadata: str, settings: Dict[str, str]):
    dir_audit = AuditDirectories(ppr, settings).audit()
    if dir_audit:
        logger.info("Directory audit successful!")
    else:
        logger.info("FAILURE: directory structure audit was unsuccessful!")
        sys.exit(1)

    audit = AuditFiles([ppr, metadata], settings).audit()
    if audit:
        logger.info("File audit successful!")
    else:
        logger.info("FAILURE: file audit was unsuccessful!")
        sys.exit(1)


def main():
    args = arg_parser().parse_args()

    path = os.getcwd()
    settings = _get_settings(pathlib.Path(path))

    if args.standard_cal is not None:
        metadata = args.standard_cal[0]
        ppr = args.standard_cal[1]

        run_audit(ppr, metadata, settings)
        if check_calibratable(metadata):
            if settings.get("useCasa"):
                launcher = CalibrationLauncher(ppr, metadata)
                launcher.setup_environment(settings)
                launcher.run()
                launcher.check_logs(parent_path=path)
                # make sure we return to the parent directory after processing
                os.chdir(path)
            else:
                logger.info("RUNNING VELA!")
                subprocess.run(["./vela", "--standard-cal", metadata, ppr])
        else:
            logger.error("ERROR: Provided SPL is not type execution block!")
            sys.exit(1)

    elif args.standard_img is not None:
        metadata = args.standard_img[0]
        ppr = args.standard_img[1]

        run_audit(ppr, metadata, settings)
        if check_imagable(metadata):
            if settings.get("useCasa"):
                launcher = ImagingLauncher(ppr, metadata)
                launcher.setup_environment(settings)
                launcher.run()
                launcher.check_logs(parent_path=path)
                # return to parent directory after processing
                os.chdir(path)
            else:
                logger.info("RUNNING VELA!")
                subprocess.run(["./vela", "--standard-img", metadata, ppr])
        else:
            logger.error("ERROR: CMS information missing or incorrect!")
            sys.exit(1)
